# CSS Line Grid Module Level 1

## Editor’s Draft, 1 January 1970

Specification Metadata

This version:
  https://drafts.csswg.org/css-line-grid/
Latest published version:
  https://www.w3.org/TR/css-line-grid-1/
Previous Versions:
  https://www.w3.org/TR/2014/WD-css-line-grid-1-20140916/
Editors:
  Elika Etemad (Invited Expert)
  Koji Ishii (Invited Expert)
  Alan Stearns (Adobe Systems, Inc.)
Issue Tracking:
  GitHub Issues
Suggest an Edit for this Spec:
  GitHub Editor

Copyright © 1970 W3C® (MIT, ERCIM, Keio, Beihang). W3C liability, trademark and permissive document license rules apply.

## Abstract

This module contains CSS features for aligning content to a baseline grid.

CSS is a language for describing the rendering of structured documents (such as HTML and XML) on screen, on paper, etc.

## Status of this document

This is a public copy of the editors’ draft. It is provided for discussion only and may change at any moment. Its publication here does not imply endorsement of its contents by W3C. Don’t cite this document other than as work in progress.

GitHub Issues are preferred for discussion of this specification. When filing an issue, please put the text “css-line-grid” in the title, preferably like this: “[css-line-grid] …summary of comment…”. All issues and comments are archived, and there is also a historical archive.

This document was produced by the CSS Working Group.

This document was produced by a group operating under the W3C Patent Policy. W3C maintains a public list of any patent disclosures made in connection with the deliverables of the group; that page also includes instructions for disclosing a patent. An individual who has actual knowledge of a patent which the individual believes contains Essential Claim(s) must disclose the information in accordance with section 6 of the W3C Patent Policy.

This document is governed by the 1 March 2019 W3C Process Document.

## 1. Introduction

### 1.1. Background

This section is not normative.

This specification provides features to align lines and blocks to invisible grids in the document.

Aligning lines and blocks to grids provides the following benefits:

- Vertical rhythm is kept for better readability.
- Lines are aligned between columns in multi-column documents.
- The top and the bottom margins of pictures are made equal, while keeping the vertical rhythm of text before and after the pictures.
- Layout lines are at the same position on every page in paged media. Keeping the position of the bottom line of a page has benefits for design and readability. This also improves the readability of duplex printing, two pages spreads, and displaying on slow display devices like e-ink.
- East Asian layouts require vertical rhythm more often than other scripts do, even in single column, non-paged media documents, as defined in [JLREQ].

There are several types of objects in a document that can break the vertical rhythm. Examples include lines with different sizes of text, pictures, and tables.

Vertical rhythm kept through pictures and different size of text in a multi-column document.

Large text wraps within line grids.

When a different size of text, such as a headings wraps, it is usually aligned to grids as a block and the lines within the block do not align.

Sidenotes (and footnotes for that matter) are often set at a smaller size than the basic text. This smaller text should still line up with the basic text. Authors can try to achieve this effect by calculating appropriate font-size, line-height, and margins*, but lack the proper tools to get the baselines to align.

Even if the author controls all this, the baselines won’t align. And careful calculations can be thrown off by user stylesheets.

Sidenotes are set at a smaller size, and baselines don’t align.

Sidenote with baselines aligned to the body text.

East Asian layouts may require width be a multiple of em without fractions.

East Asian layouts may require grid-like features in inline progression direction as well.

It is often desirable in East Asian layouts to make the line width a multiple of em without fractions. Because most East Asian characters have 1em advance and most East Asian documents are justified, this minimizes cases where justification needs to expand character spacing.

This module provides the following capabilities:

- Defining grids in the line progression direction.
- Controlling how lines and blocks align to the grids.

It is important to control these capabilities independently, so that, for example, aligning to grids can be turned off for tables, but can then be turned back on for aligning the following text to the grids.

### 1.2. Module Interactions

This module extends the line box model defined in [CSS2] sections 9.4.2 and 10.8.

### 1.3. Values

This specification follows the CSS property definition conventions from [CSS2]. Value types not defined in this specification are defined in CSS Values & Units [CSS-VALUES-3]. Other CSS modules may expand the definitions of these value types.

In addition to the property-specific values listed in their definitions, all properties defined in this specification also accept the CSS-wide keywords keywords as their property value. For readability they have not been repeated explicitly.

## 2. Defining a Line Grid: the line-grid property

Name: | line-grid
Value: | match-parent | create
Initial: | match-parent
Applies to: | block, flex and grid containers
Inherited: | no
Percentages: | N/A
Computed value: | specified keyword
Canonical order: | per grammar
Animation type: | discrete

Specifies whether this box creates a new baseline grid for its descendants or uses the same baseline grid as its parent. (Each box always has an associated line grid. However, whether a box or its contents snap to a line grid is determined by line-snap and box-snap.)

The values of this property have the following meanings:

match-parent
  Box assumes the line grid of its parent if its writing mode is the same as its parent. If the box has a writing mode that is different than its parent, then the box creates a new line grid as create below.
create
  Box creates a new line grid using its own font and line layout settings, including any adjustment to the line height caused by the line-height-step property. The line grid consists of a series of horizontal lines corresponding to all the baselines (alphabetic, text-top, text-bottom, mathematic, central, hanging, etc.) and to the line-over and line-under edges, positioned where they would fall if the contents of this element consisted entirely of line boxes filled with text (no sub-elements) using the first available font. If the box is paginated, the line grid is restarted on each page; since line boxes cannot be fragmented, no page begins with the bottom part of a line’s grid.

The initial containing block establishes a line grid using the font and line layout settings of the root element. If the root element (which has no parent) has a value of match-parent, then it adopts the line grid of the initial containing block.

In paged media, if the page context applying to a particular page specifies line-grid: create, then a line grid for that page is established using the page context’s font and line layout settings applied to the page box (including its margins), and for layout within that page the initial containing block is assumed to have adopted this line grid.

Since the initial value of line-grid is match-parent, by default a line grid is automatically created using the root element’s font and line height settings to fill the initial containing block / page area. When all pages of a document are the same size, this will result in a consistent line grid that exactly aligns to the top of each page area.

It is sometimes appropriate for the line grid to begin with the content, after the root element’s margins/padding/border and/or other header content. To do this, the root element (or some other element) can be told to establish the line grid.

    main { line-grid: create; }

If this line grid is then paginated, the line grid will start at the top of each page area after the first.

In other cases, it’s desirable for all pages of a document share a common line grid, even if page margins, border and/or padding change. The following rule will instead fill the page box area including the page margin/padding/borders with the line grid:

    @page { line-grid: create; }

As this line grid is paginated, it will start at the top of each page box after the first; however it might not align to the top of the page area (where the content would normally begin).

The names of these values is currently up for debate. Current suggestions for match-parent include match-parent and normal; those for create include create and new.

The original proposal for line grids allowed an element to create a named grid. This property could still be extended to do this in the future.

There might need to be an offset for more complicated designs. How to set this offset is problematic: usually it’s not a fixed length, but the distance to clear some header content. This could be added to a later level of css-line-grid.

## 3. Snapping to a Grid

The line-snap and box-snap properties cause line boxes and block boxes to shift position in order to align to the line grid. Shifting boxes in this way affects layout – it is not merely a display translation. If a line box is shifted, then subsequent line boxes will be laid out using the new shifted position as input to their line stacking rules. If a block box is shifted, then subsequent boxes will be laid out using the new shifted position as input.

### 3.1. Snapping Line Boxes: the line-snap property

Name: | line-snap
Value: | none | baseline | contain
Initial: | none
Applies to: | block container elements
Inherited: | yes
Percentages: | N/A
Computed value: | specified keyword
Canonical order: | per grammar
Animation type: | discrete

This property applies to all the line boxes directly contained by the element, and, when not none, causes each line box to shift (usually downward, possibly by zero) until it snaps to the line grid specified by line-grid. (The unshifted position is the position that would be determined by normal line stacking rules, with consideration of any new controls defined by other modules such as [CSS3LINE].)

Values have the following meanings:

none
  Line boxes do not snap to the grid; they stack normally.
baseline
  The dominant baseline snaps with the matching baseline on the line grid applying to the element.
contain
  Two baselines are used to align the line box: the line box is snapped so that its central baseline is centered between one of the line grid’s text-over-edge baselines and a subsequent (but not necessarily consecutive) text-under-edge baseline.

Line boxes almost always shift downward (towards the block-end direction) when snapping to a line grid. Here there are three lines with 20px line-height and line-snap:baseline that should snap to a 30px line grid. Each line box shifts down so that the baselines align with the grid lines.

Before line snapping
After line snapping

In the figures below, there are two additional lines from h3 elements with line-snap:none. These lines do not shift to align to the grid, but their positions can change based on the shifting of lines around them. In this example, lines 1 2 and 3 shift down to snap their baselines to the grid lines, and line B has normal line box placement just below the line above.

Before line snapping
After line snapping

### 3.2. Snapping Block Boxes: the box-snap property

This is a rough draft of trying to solve the box-snapping problem.

Some optional box values (margin-box, border-box) could be added to the before and after values to allow snapping various box model edges to the line grid.

An auto value could be useful - one that defaults to center, but snaps to before if it’s the first block in a fragment container, and snaps to after if it’s the last block in a fragment container.

Name: | box-snap
Value: | none | block-start | block-end | center | baseline | last-baseline
Initial: | none
Applies to: | block-level boxes and internal table elements except table cells
Inherited: | yes
Percentages: | N/A
Computed value: | as specified
Canonical order: | per grammar
Animation type: | discrete

Specifies how the block is snapped to the baseline grid.

Values have the following meanings:

none
  The block is not snapped to any grid.
block-start
  The block-start edge is snapped to the nearest grid line.
block-end
  The block-end edge is snapped to the nearest grid line.
center
  The block is centered between one of the baseline grid’s text-over baselines and a subsequent (but not necessarily consecutive) text-under baseline.
baseline
  The first line box’s dominant baseline is snapped to the nearest grid line.
last-baseline
  The last line box’s dominant baseline is snapped to the nearest grid line.

Snapping block boxes always uses the line grid of the box parent (a block’s own line grid has no effect on box snapping). If the box is an empty block that could be collapsed through, then this property has no effect. [CSS2]

To snap a block-level element to a grid line, take the unsnapped result of layout (including margin collapsing, clearance, etc.) as the block’s initial position, then shift the block downward (towards the block-end direction) until it snaps to the line grid.

For the block-start and block-end values, either the text-over-edge or text-under-edge baseline is chosen: whichever one is on the matching side of the central baseline. For example, when snapping the block-start edge in horizontal writing mode, the text-over-edge is chosen. In some cases the text-under-edge might be used instead for the block-start edge: for example, when the writing mode of the line grid doesn’t match that of the affected element, or when due to the text-orientation settings the text-under-edge corresponds to the block-start edge.

When applied to table row group and table row boxes, box-snap only affects the before and after edges, and only if those edges are not at the beginning or end of the table, respectively. To snap a before edge on a table row or row group, the preceding row’s height is increased. To snap an after edge on a table row or row group, the affected row’s height is increased.

When applied to table column group and table column boxes, 'box-snap’only affects the start and end edges, and only if those edges are not at the start or end of the table, respectively. How the space is redistributed among columns to satisfy snapping constraints is not defined, however:

- In an auto-sized table no column may be smaller than its minimum content width.
- The resulting table must not exceed its original measure if it had a non-auto measure.
- The adjusted widths must not cause the table to overflow its containing block any more than it would with box-snap: none.

To satisfy these constraints, some column edges may remain unsnapped.

Going forward, we could add a 2-d box-snap with more controls and to also extend it to handle snapping to an explicit layout grid (not just the baseline grid). Current thoughts are for box-snap to break out into longhands; the current block-snap property should thus be a subset of its expected future values. Below is an outline of the structure. (This may, of course, change.)

    box-snap
      +- block-snap
      |     +- block-snap-edge
      |     |     +- block-start-snap-edge: <shape-box>
      |     |     +- block-end-snap-edge: <shape-box>
      |     +- block-snap-align: <content-position>
      |     +- block-snap-grid: text-grid | <custom-ident> /* name of grid/gridlines */
      +- inline-snap
            +- inline-snap-edge
            |     +- inline-start-snap-edge: <shape-box>
            |     +- inline-end-snap-edge: <shape-box>
            +- inline-snap-align: <content-position>
            +- inline-snap-grid: text-grid | <custom-ident> /* name of grid/gridlines */

    where <shape-box> includes
      margin-box | border-box | half-border-box | padding-box | content-box
    and <content-position> includes
      start | end | stretch | first-baseline | last-baseline | center

## 4. Snapping Details

### 4.1. Interacting with other alignments

An element can have additional block layout constraints (such as centering or box-snap) that can complicate line snapping. In these cases, implementations must produce the same result as the steps below:

- Line snap as if the layout constraint does not apply
- Apply the layout constraint to the block
- Adjust exactly one line box shift or space at the beginning or end of the block such that all of the snapping lines meet the line grid while maintaining the block layout constraint. The single adjustment chosen must be the smallest shift needed to achieve the result without causing overflow.

A block containing lines to snap might not be top-aligned within its container. In the figures below, the block containing the elements is centered. In a centered situation, you have to align baselines while maintaining centering. This can be done in two shift-and-center steps.

First, shift the snapping lines as if the block was top-aligned (as in figure 9 above), then remove the shift for the very first snapping line. After removing the first shift, try centering the block. This is almost certain to throw the baseline alignment off. You can see one such result in the partial shifting figure below.

Second, measure the distance from the first snapped line’s baseline to the grid lines above and below, looking for the closest grid line to that baseline.

If the closest grid line is in the block-start direction, then add space below the last line in the block equal to twice that distance. Then the block is centered again, which will align all of the snapped lines to the grid.

Partial shifting
Full line snapping

If the closest grid line is in the block-end direction, then the first snapped line is shifted downward by twice that distance. Then the block is centered again, which will again align all of the snapped lines to the grid.

Partial shifting
Full line snapping

An end-aligned block also uses two steps, but is simpler than the centered case.

First, shift the snapping lines as if there were no end-alignment (as in figure 9 above), then end-align the block.

Second, shift the entire block contents upwards until the last snapped line aligns to a grid line. In this example, the shift is very minor.

Before snapping
After line snapping

### 4.2. Leading adjustments

In some cases lines of equal line height will not align perfectly to a baseline grid: this happens, for example, when fonts (of the same size) with different baseline tables are mixed on a line. In these cases, removing top half-leading can sometimes be the correct thing to do, but must be limited to the amount of baseline discrepancy so as not to trigger in other cases.

Details to follow. It’s complicated when you have more than one font-size or line-height in a single line box.

## Acknowledgments

This module was made possible by the advice and contributions of Tab Atkins, Dave Cramer, Dave Hyatt, Bem Jones-Bey, Håkon Wium Lie, Shinyu Murakami, Liam Quin, and the CSS Working Group members.

## Change Log

### Since September 16th 2014

- Refined ICB/root/@page grid creation
- Added flex and grid container applicability for line-grid
- Added normative text for line-snap examples
- Added @page and root element line grid handling
- Made the dependency on line-height-step explicit
- match-parent creates new line grid for orthoganal flows

### Since April 3rd 2014

- Added line-snapping examples
- Modified box-snap values
- Removed line-slack
- Removed grid offset
- Removed grid units
- Removed 2d box snapping
- Removed rounding functions

## Conformance

### Document conventions

Conformance requirements are expressed with a combination of descriptive assertions and RFC 2119 terminology. The key words “MUST”, “MUST NOT”, “REQUIRED”, “SHALL”, “SHALL NOT”, “SHOULD”, “SHOULD NOT”, “RECOMMENDED”, “MAY”, and “OPTIONAL” in the normative parts of this document are to be interpreted as described in RFC 2119. However, for readability, these words do not appear in all uppercase letters in this specification.

All of the text of this specification is normative except sections explicitly marked as non-normative, examples, and notes. [RFC2119]

Examples in this specification are introduced with the words “for example” or are set apart from the normative text with `class="example"`, like this:

This is an example of an informative example.

Informative notes begin with the word “Note” and are set apart from the normative text with `class="note"`, like this:

Note, this is an informative note.

Advisements are normative sections styled to evoke special attention and are set apart from other normative text with `<strong class="advisement">`, like this: UAs MUST provide an accessible alternative.

### Conformance classes

Conformance to this specification is defined for three conformance classes:

style sheet
  A CSS style sheet.
renderer
  A UA that interprets the semantics of a style sheet and renders documents that use them.
authoring tool
  A UA that writes a style sheet.

A style sheet is conformant to this specification if all of its statements that use syntax defined in this module are valid according to the generic CSS grammar and the individual grammars of each feature defined in this module.

A renderer is conformant to this specification if, in addition to interpreting the style sheet as defined by the appropriate specifications, it supports all the features defined by this specification by parsing them correctly and rendering the document accordingly. However, the inability of a UA to correctly render a document due to limitations of the device does not make the UA non-conformant. (For example, a UA is not required to render color on a monochrome monitor.)

An authoring tool is conformant to this specification if it writes style sheets that are syntactically correct according to the generic CSS grammar and the individual grammars of each feature in this module, and meet all other conformance requirements of style sheets as described in this module.

### Requirements for Responsible Implementation of CSS

The following sections define several conformance requirements for implementing CSS responsibly, in a way that promotes interoperability in the present and future.

#### Partial Implementations

So that authors can exploit the forward-compatible parsing rules to assign fallback values, CSS renderers must treat as invalid (and ignore as appropriate) any at-rules, properties, property values, keywords, and other syntactic constructs for which they have no usable level of support. In particular, user agents must not selectively ignore unsupported property values and honor supported values in a single multi-value property declaration: if any value is considered invalid (as unsupported values must be), CSS requires that the entire declaration be ignored.

#### Implementations of Unstable and Proprietary Features

To avoid clashes with future stable CSS features, the CSSWG recommends following best practices for the implementation of unstable features and proprietary extensions to CSS.

#### Implementations of CR-level Features

Once a specification reaches the Candidate Recommendation stage, implementers should release an unprefixed implementation of any CR-level feature they can demonstrate to be correctly implemented according to spec, and should avoid exposing a prefixed variant of that feature.

To establish and maintain the interoperability of CSS across implementations, the CSS Working Group requests that non-experimental CSS renderers submit an implementation report (and, if necessary, the testcases used for that implementation report) to the W3C before releasing an unprefixed implementation of any CSS features. Testcases submitted to W3C are subject to review and correction by the CSS Working Group.

Further information on submitting testcases and implementation reports can be found from on the CSS Working Group’s website at http://www.w3.org/Style/CSS/Test/. Questions should be directed to the public-css-testsuite@w3.org mailing list.

## Index

### Terms defined by this specification

- baseline
  - value for box-snap, in §3.2
  - value for line-snap, in §3.1
- block-end, in §3.2
- block-start, in §3.2
- box-snap, in §3.2
- center, in §3.2
- contain, in §3.1
- create, in §2
- last-baseline, in §3.2
- Line Grid, in §2
- line-grid, in §2
- line-snap, in §3.1
- match-parent, in §2
- none
  - value for box-snap, in §3.2
  - value for line-snap, in §3.1

https://drafts.csswg.org/css-align-3/#typedef-content-position Referenced in:

- 3.2. Snapping Block Boxes: the box-snap property (2) (3)

https://drafts.csswg.org/css-display-3/#block-container Referenced in:

- 3.1. Snapping Line Boxes: the line-snap property

https://drafts.csswg.org/css-rhythm-1/#propdef-line-height-step Referenced in:

- 2. Defining a Line Grid: the line-grid property

https://drafts.csswg.org/css-shapes-1/#typedef-shape-box Referenced in:

- 3.2. Snapping Block Boxes: the box-snap property (2) (3) (4) (5)

https://drafts.csswg.org/css-values-4/#identifier-value Referenced in:

- 3.2. Snapping Block Boxes: the box-snap property (2)

https://drafts.csswg.org/css-values-4/#css-wide-keywords Referenced in:

- 1.3. Values

https://drafts.csswg.org/css-values-4/#comb-one Referenced in:

- 2. Defining a Line Grid: the line-grid property
- 3.1. Snapping Line Boxes: the line-snap property (2)
- 3.2. Snapping Block Boxes: the box-snap property (2) (3) (4) (5)

https://drafts.csswg.org/css-writing-modes-4/#propdef-text-orientation Referenced in:

- 3.2. Snapping Block Boxes: the box-snap property

https://drafts.csswg.org/css-writing-modes-4/#writing-mode Referenced in:

- 2. Defining a Line Grid: the line-grid property (2)

### Terms defined by reference

- [css-align-3] defines the following terms:
  - <content-position>
- [css-display-3] defines the following terms:
  - block container
- [css-rhythm-1] defines the following terms:
  - line-height-step
- [css-shapes-1] defines the following terms:
  - <shape-box>
- [css-values-4] defines the following terms:
  - <custom-ident>
  - css-wide keywords
  - |
- [css-writing-modes-4] defines the following terms:
  - text-orientation
  - writing mode

## References

### Normative References

[CSS-ALIGN-3]
  Elika Etemad; Tab Atkins Jr.. CSS Box Alignment Module Level 3. 6 December 2018. WD. URL: https://www.w3.org/TR/css-align-3/
[CSS-DISPLAY-3]
  Tab Atkins Jr.; Elika Etemad. CSS Display Module Level 3. 28 August 2018. CR. URL: https://www.w3.org/TR/css-display-3/
[CSS-RHYTHM-1]
  Koji Ishii; Elika Etemad. CSS Rhythmic Sizing. 2 March 2017. WD. URL: https://www.w3.org/TR/css-rhythm-1/
[CSS-SHAPES-1]
  Vincent Hardy; Rossen Atanassov; Alan Stearns. CSS Shapes Module Level 1. 20 March 2014. CR. URL: https://www.w3.org/TR/css-shapes-1/
[CSS-VALUES-3]
  Tab Atkins Jr.; Elika Etemad. CSS Values and Units Module Level 3. 31 January 2019. CR. URL: https://www.w3.org/TR/css-values-3/
[CSS-VALUES-4]
  Tab Atkins Jr.; Elika Etemad. CSS Values and Units Module Level 4. 31 January 2019. WD. URL: https://www.w3.org/TR/css-values-4/
[CSS-WRITING-MODES-4]
  Elika Etemad; Koji Ishii. CSS Writing Modes Level 4. 24 May 2018. CR. URL: https://www.w3.org/TR/css-writing-modes-4/
[CSS2]
  Bert Bos; et al. Cascading Style Sheets Level 2 Revision 1 (CSS 2.1) Specification. 7 June 2011. REC. URL: https://www.w3.org/TR/CSS2/
[RFC2119]
  S. Bradner. Key words for use in RFCs to Indicate Requirement Levels. March 1997. Best Current Practice. URL: https://tools.ietf.org/html/rfc2119

### Informative References

[CSS3LINE]
  Dave Cramer; Elika Etemad; Steve Zilles. CSS Inline Layout Module Level 3. 8 August 2018. WD. URL: https://www.w3.org/TR/css-inline-3/
[JLREQ]
  Yasuhiro Anan; et al. Requirements for Japanese Text Layout. 3 April 2012. NOTE. URL: https://www.w3.org/TR/jlreq/

## Property Index

Name | Value | Initial | Applies to | Inh. | %ages | Animation type | Canonical order | Computed value
--- | --- | --- | --- | --- | --- | --- | --- | ---
box-snap | none | block-start | block-end | center | baseline | last-baseline | none | block-level boxes and internal table elements except table cells | yes | N/A | discrete | per grammar | as specified
line-grid | match-parent | create | match-parent | block, flex and grid containers | no | N/A | discrete | per grammar | specified keyword
line-snap | none | baseline | contain | none | block container elements | yes | N/A | discrete | per grammar | specified keyword

## Issues Index

The names of these values is currently up for debate. Current suggestions for match-parent include match-parent and normal; those for create include create and new. ↵
This is a rough draft of trying to solve the box-snapping problem. ↵
Some optional box values (margin-box, border-box) could be added to the before and after values to allow snapping various box model edges to the line grid. ↵
An auto value could be useful - one that defaults to center, but snaps to before if it’s the first block in a fragment container, and snaps to after if it’s the last block in a fragment container. ↵
Going forward, we could add a 2-d box-snap with more controls and to also extend it to handle snapping to an explicit layout grid (not just the baseline grid). Current thoughts are for box-snap to break out into longhands; the current block-snap property should thus be a subset of its expected future values. Below is an outline of the structure. (This may, of course, change.)

    box-snap
      +- block-snap
      |     +- block-snap-edge
      |     |     +- block-start-snap-edge: <shape-box>
      |     |     +- block-end-snap-edge: <shape-box>
      |     +- block-snap-align: <content-position>
      |     +- block-snap-grid: text-grid | <custom-ident> /* name of grid/gridlines */
      +- inline-snap
            +- inline-snap-edge
            |     +- inline-start-snap-edge: <shape-box>
            |     +- inline-end-snap-edge: <shape-box>
            +- inline-snap-align: <content-position>
            +- inline-snap-grid: text-grid | <custom-ident> /* name of grid/gridlines */

    where <shape-box> includes
      margin-box | border-box | half-border-box | padding-box | content-box
    and <content-position> includes
      start | end | stretch | first-baseline | last-baseline | center

↵
Details to follow. It’s complicated when you have more than one font-size or line-height in a single line box. ↵
#line-grid Referenced in:

- 2. Defining a Line Grid: the line-grid property (2) (3) (4) (5) (6) (7) (8) (9) (10) (11) (12) (13) (14) (15) (16) (17) (18) (19) (20)
- 3. Snapping to a Grid
- 3.1. Snapping Line Boxes: the line-snap property (2) (3) (4)
- 3.2. Snapping Block Boxes: the box-snap property (2) (3) (4) (5)
- 4.1. Interacting with other alignments

#propdef-line-grid Referenced in:

- 2. Defining a Line Grid: the line-grid property (2) (3)
- 3.1. Snapping Line Boxes: the line-snap property

#valdef-line-grid-match-parent Referenced in:

- 2. Defining a Line Grid: the line-grid property (2)
- Since September 16th 2014

#valdef-line-grid-create Referenced in:

- 2. Defining a Line Grid: the line-grid property

#propdef-line-snap Referenced in:

- 2. Defining a Line Grid: the line-grid property
- 3. Snapping to a Grid
- 3.1. Snapping Line Boxes: the line-snap property

#propdef-box-snap Referenced in:

- 2. Defining a Line Grid: the line-grid property
- 3. Snapping to a Grid
- 3.2. Snapping Block Boxes: the box-snap property (2) (3)
- 4.1. Interacting with other alignments

#valdef-box-snap-block-start Referenced in:

- 3.2. Snapping Block Boxes: the box-snap property

#valdef-box-snap-block-end Referenced in:

- 3.2. Snapping Block Boxes: the box-snap property